/* Copyright © 2011 Brandon L Black <blblack@gmail.com>
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _GDNSD_CONF_H
#define _GDNSD_CONF_H

#include "config.h"
#include "gdnsd.h"
#include "zscan.h"

typedef struct {
    const char* name;
    gdnsd_full_config_cb_t full_config;
    gdnsd_map_resource_dyna_cb_t map_resource_dyna;
    gdnsd_map_resource_dync_cb_t map_resource_dync;
    gdnsd_pre_privdrop_cb_t pre_privdrop;
    gdnsd_pre_run_cb_t pre_run;
    gdnsd_iothread_init_cb_t iothread_init;
    gdnsd_resolve_dynaddr_cb_t resolve_dynaddr;
    gdnsd_resolve_dyncname_cb_t resolve_dyncname;
    gdnsd_exit_cb_t exit;
} plugin_t;

typedef struct {
    const char* name;
    const char* file;
    unsigned esep_bits;
    unsigned esep_val;
    // The above are set directly from the config
    // The next 2 below are dynamically set during a
    //  series of passes over the listener/key config,
    //  and are ultimately used in the calculation
    //  of this structure's final sep_mask and the
    //  per-dnsaddr udp/tcp _sep_val
    unsigned num_socks;
    unsigned cur_sock;
    uint32_t sep_mask;
    // load_curve_key_data() populates this after
    //  daemonization but before chroot(), and then
    //  the I/O threads later copy from here.
    const uint8_t* secret;
} curvekey_t;

typedef struct {
    curvekey_t* key;
    uint32_t udp_sep_val;
    uint32_t tcp_sep_val;
} dnsaddr_key_t;

typedef struct {
    anysin_t addr;
    int      udp_sock;
    int      tcp_sock;
    bool     tcp_disabled;
    bool     udp_need_late_bind;
    bool     tcp_need_late_bind;
    unsigned late_bind_secs;
    unsigned tcp_timeout;
    unsigned tcp_clients_per_socket;
    unsigned udp_threadnum;
    unsigned tcp_threadnum;
    unsigned num_curve_keys;
    unsigned udp_kc_bits;
    unsigned tcp_kc_bits;
    unsigned udp_recv_width;
    unsigned udp_sndbuf;
    unsigned udp_rcvbuf;
    dnsaddr_key_t* curve_keys;
} dns_addr_t;

typedef struct {
    zoneinfo_t* zones;
    dns_addr_t* dns_addrs;
    anysin_t*   http_addrs;
    plugin_t*   plugins;
    const char*     pidfile;
    const char*     username;
    const char*     chroot_path;
    const char*     dns_port;
    const char*     http_port;
    curvekey_t*     curve_keys;
    bool     include_optional_ns;
    bool     realtime_stats;
    bool     lock_mem;
    bool     disable_text_autosplit;
    bool     strict_data;
    bool     edns_client_subnet;
    int      priority;
    unsigned num_curve_keys;
    unsigned zones_default_ttl;
    unsigned log_stats;
    unsigned max_http_clients;
    unsigned http_timeout;
    unsigned num_zones;
    unsigned num_dns_addrs;
    unsigned num_http_addrs;
    unsigned num_plugins;
    unsigned num_io_threads;
    unsigned max_response;
    unsigned outer_max_response;
    unsigned max_cname_depth;
} global_config_t;

extern global_config_t gconfig;

F_NONNULL
void conf_load(const char* cfg_file);

F_NONNULL
char* make_cf_fn(const char* dir, const char* fn);

// retval indicates we need runtime CAP_NET_BIND_DEVICE
bool dns_lsock_init(void);

// utility function, must be AF_INET or AF_INET6 already,
//  used by dnsio_udp
F_NONNULL F_PURE
bool is_any_addr(const anysin_t* asin);

#undef _RC
#endif // _GDNSD_CONF_H
