/* Copyright © 2011 Brandon L Black <blblack@gmail.com>
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _GDNSD_PLUGAPI_H
#define _GDNSD_PLUGAPI_H

// for uint8_t
#include <inttypes.h>

// For struct ev_loop
#include <gdnsd-ev.h>

// For vscf_data_t
#include <gdnsd-vscf.h>

// For anysin_t
#include <gdnsd-net.h>

// For monio_list_t
#include <gdnsd-monio.h>

/***
 * Plugin API version, bumped on any change that's not backwards-compat.
 * This is hardcoded as a the return value of plugin_foo_get_api_version()
 *   in gdnsd-plugin.h, and also compiled into the gdnsd code.  The two
 *   values are compared at plugin load time to ensure that plugin code
 *   which doesn't match the API of the gdnsd binary is not allowed.
 ***/
#define GDNSD_PLUGIN_API_VERSION 6

/*** Data Types ***/

typedef struct {
    anysin_t dns_source;
    anysin_t edns_client;
    unsigned edns_client_mask; // if zero, edns_client is invalid, do not parse it
} client_info_t;

// Result structure for dynamic address resolution plugins
typedef struct {
    uint32_t ttl;
    unsigned edns_scope_mask;
    unsigned count_v4;
    unsigned count_v6;
    uint32_t addrs_v4[64];
    uint8_t  addrs_v6[64 * 16];
} dynaddr_result_t;

// Result structure for dynamic cname resolution plugins
typedef struct {
    uint32_t ttl;
    unsigned edns_scope_mask;
    uint8_t dname[256];
} dyncname_result_t;

/**** Typedefs for plugin callbacks ****/

typedef unsigned (*gdnsd_apiv_cb_t)(void);
typedef monio_list_t* (*gdnsd_load_config_cb_t)(const vscf_data_t* pc);
typedef unsigned (*gdnsd_map_resource_dyna_cb_t)(const char* resname);
typedef unsigned (*gdnsd_map_resource_dync_cb_t)(const char* resname, const uint8_t* origin);
typedef void (*gdnsd_full_config_cb_t)(unsigned num_threads);
typedef void (*gdnsd_pre_privdrop_cb_t)(void);
typedef void (*gdnsd_pre_run_cb_t)(struct ev_loop* loop);
typedef void (*gdnsd_iothread_init_cb_t)(unsigned threadnum);
typedef void (*gdnsd_resolve_dynaddr_cb_t)(unsigned threadnum, unsigned resnum, const client_info_t* cinfo, dynaddr_result_t* result);
typedef void (*gdnsd_resolve_dyncname_cb_t)(unsigned threadnum, unsigned resnum, const uint8_t* origin, const client_info_t* cinfo, dyncname_result_t* result);
typedef void (*gdnsd_exit_cb_t)(void);

#endif // _GDNSD_PLUGINAPI_H
